use std::{collections::HashMap, iter::zip, path::Path};

use crate::{days::AdventDay, error::AdventError, input::get_lines};

pub struct Day1 {
    list1: Vec<i32>,
    list2: Vec<i32>,
    count1: HashMap<i32, u64>,
    count2: HashMap<i32, u64>
}


impl AdventDay for Day1 {
    fn new() -> Self {
        Day1 {
            list1: vec![],
            list2: vec![],
            count1: HashMap::new(),
            count2: HashMap::new()
        }
    }

    fn read_input(&mut self) -> Result<(),AdventError> {
        for line in get_lines(Path::new("resources/input1.txt")) {
            let line = line?;
            let mut s = line.split("   ");
            let l = s.next().unwrap().parse::<i32>()?;
            self.list1.push(l);
            if self.count1.contains_key(&l) {
                *self.count1.get_mut(&l).unwrap() += 1;
            }
            else {
                self.count1.insert(l, 1);
            }
            let r = s.next().unwrap().parse::<i32>()?;
            self.list2.push(r);
            if self.count2.contains_key(&r) {
                *self.count2.get_mut(&r).unwrap() += 1;
            }
            else {
                self.count2.insert(r, 1);
            }
        }
        Ok(())
    }
    
    fn puzzle1(&mut self) -> Result<u64, AdventError> {
        
        self.list1.sort();
        self.list2.sort();

        Ok(zip(self.list1.clone(), self.list2.clone())
        .map(|(l, r)| {
            (l - r).abs() as u64
        })
        .sum::<u64>())
    }
    
    fn puzzle2(&mut self) -> Result<u64, AdventError> {
        Ok(self.count1.keys().into_iter().map(
            |k| {
            let s = *k as u64;
            if self.count2.contains_key(k) {
                s * self.count1[k] * self.count2[k]
            }
            else {
                0
            }
    }).sum())
    }
}